<?php
	
	use Tygh\Enum\ProductFeatures;
	use Tygh\Enum\YesNo;
	use Tygh\Exceptions\DeveloperException;
	use Tygh\Tools\SecurityHelper;
	use Tygh\Navigation\LastView;
	use Tygh\Registry;
	use Tygh\Languages\Languages;
	use Tygh\Tygh;
	use Tygh\Addons\Switips;
	use Tygh\Enum\NotificationSeverity;
	use Tygh\Enum\ImagePairTypes;
	
	if (!defined('BOOTSTRAP')) { die('Access denied'); }
	
	function fn_switips_init_layout($params, $layout) {
		if (!isset($_COOKIE['switips_uid'])) {
			$_COOKIE['switips_uid'] = '';
			if (!empty($_GET['uid'])) {
				$uid = trim($_GET['uid']);
				$pattern = "#WP[\d]{10}#";
				$is_switips = preg_match($pattern, $uid);
				
				if($is_switips) {
					$_COOKIE['switips_uid'] = $_GET['uid'];
					
					setcookie('switips_uid', $uid, time()+60*60*24*30);
				}
			}
		}
	}
	
	function fn_switips_change_order_status_post($order_id, $status_to, $status_from, $force_notification, $place_order, $order_info, $edp_data) {
		
		$states = [];
		
		$currency = Registry::get('addons.switips.switips_currency');
		$url = Registry::get('addons.switips.switips_url');
		$merchant_id = Registry::get('addons.switips.switips_merchant_id');
		$secret_key = Registry::get('addons.switips.switips_secret_key');
		
		$new = Registry::get('addons.switips.order_status_new');
		$paid = Registry::get('addons.switips.order_status_paid');
		$canceled = Registry::get('addons.switips.order_status_canceled');
		$confirmed = Registry::get('addons.switips.order_status_confirmed');
		
		if($new) {
			foreach($new as $key => $val) {
				$states[$key] = 'new';
			}
		}
		
		if($paid) {
			foreach($paid as $key => $val) {
				$states[$key] = 'paid';
			}
		}
		
		if($canceled) {
			foreach($canceled as $key => $val) {
				$states[$key] = 'canceled';
			}
		}
		
		if($confirmed) {
			foreach($confirmed as $key => $val) {
				$states[$key] = 'confirmed';
			}
		}
		
		$order_statuses = fn_get_statuses(STATUSES_ORDER);
		$status_arr = array();
		
		if (!empty($order_statuses)) {
			foreach ($order_statuses as $key => $field) {
				$status_arr[$key] = $field['status_id'];
			}
		}
		
		//fn_print_r(print_r($status_arr, true));
		//fn_print_r($status_to);
		
		if(array_key_exists($status_to, $status_arr) && isset($states[$status_arr[$status_to]])) {
			$uid = 0;
			if($states[$status_arr[$status_to]] === 'new') {
				$uid = $_COOKIE['switips_uid'] ? $_COOKIE['switips_uid'] : 0;
				
				if($uid) {
					$item_data = array(
						'createdon' => time(),
						'user_id' => $uid,
						'order' => $order_id,
					);
					
					$item_id = fn_switips_set_item($item_data);
				}
			}
			else {
				$order_data = fn_switips_get_item($order_id);
				
				if(!empty($order_data)) {
					$uid = isset($order_data['user_id']) && !empty($order_data['user_id']) ? $order_data['user_id'] : 0;
					
					$item_id = $order_data['item_id'];
				}
			}
			
			if($uid && $merchant_id && $secret_key) {
				
				$totalAmount = 0;
				
				foreach($order_info['products'] as $k => $v) {
					$amount = 0;
					$amountPercent = 0;
					
					$amountTmp = 0;
					$amountTmpPercent = 0;
					$productAmount = array();
					$productAmountPercent = array();
					
					$product_commissions = fn_switips_get_entity_commission($v['product_id'], 'products');
					
					$productAllAmount = fn_switips_get_entity_amount($product_commissions);
					
					if(isset($productAllAmount['percent'])) {
						$productAmountPercent = $productAllAmount['percent'];
					}
					
					if(isset($productAllAmount['absolute'])) {
						$productAmount = $productAllAmount['absolute'];
					}
					
					if(!empty($productAmount) || !empty($productAmountPercent)) {
						if(!empty($productAmount)) {
							$amountTmp = max($productAmount);
						}
						
						if(!empty($productAmountPercent)) {
							$amountTmpPercent = max($productAmountPercent);
						}
					}
					else {
						$catAmount = array();
						$catAmountPercent = array();
						$categories = fn_switips_get_product_categories($v['product_id']);
						$categories_ids = array();
						
						//fn_print_r(print_r($categories, true));
						
						foreach ($categories as $category) {
							$categories_ids[] = $category['category_id'];
						}
						
						//fn_print_r(print_r($categories_ids, true));
						
						$all_categories = fn_get_categories_list_with_parents($categories_ids);
						
						//fn_print_r(print_r($all_categories, true));
						
						foreach($all_categories as $current_category => $category_data) {
							$category_commissions = fn_switips_get_entity_commission($current_category, 'categories');
							if(!empty($category_commissions)) {
								$tmpCatAmount = fn_switips_get_entity_amount($category_commissions);
								
								if(isset($tmpCatAmount['percent'])) {
									$catAmountPercent = array_merge($catAmount, $tmpCatAmount['percent']);
								}
								
								if(isset($tmpCatAmount['absolute'])) {
									$catAmount = array_merge($catAmount, $tmpCatAmount['absolute']);
								}
							}
							
							if(isset($category_data['parents'])) {
								foreach($category_data['parents'] as $parent) {
									//fn_print_r($parent['category_id']);
									$category_commissions = fn_switips_get_entity_commission($parent['category_id'], 'categories');
									if(!empty($category_commissions)) {
										$tmpCatAmount = fn_switips_get_entity_amount($category_commissions);
										
										if(isset($tmpCatAmount['percent'])) {
											$catAmountPercent = array_merge($catAmount, $tmpCatAmount['percent']);
										}
										
										if(isset($tmpCatAmount['absolute'])) {
											$catAmount = array_merge($catAmount, $tmpCatAmount['absolute']);
										}
									}
									//fn_print_r(print_r($category_commissions, true));
								}
							}
						}
						
						if(!empty($catAmount) || !empty($catAmountPercent)) {
							if(!empty($catAmount)) {
								$amountTmp = max($catAmount);
							}
							
							if(!empty($catAmountPercent)) {
								$amountTmpPercent = max($catAmountPercent);
							}
						}
					}
					
					if($amountTmp) {
						$amount = $amountTmp * $v['amount'];
					}
					
					if($amountTmpPercent) {
						$amountPercent = (($amountTmpPercent / 100) * $v['price']) * $v['amount'];
					}
					
					$totalAmount += max(array($amount, $amountPercent));
				}
				
				if($totalAmount) {
					
					$cart_cost = $order_info['total'] - $order_info['shipping_cost'];
					
					$status_data = array(
						'item' => $item_id,
						'status' => $status_arr[$status_to],
						'createdon' => time(),
						'amount' => $totalAmount,
						'cost' => $cart_cost
					);
					
					fn_switips_set_status($status_data);
					
					$switips_core = new Switips($url, [$currency], $states);
					
					$args = [
						'merchant_id' => $merchant_id,
						'user_id' => $uid,
						'campaign_id' => 0,
						'category_id' => 0,
						'transaction_id' => $order_id,
						'transaction_amount' => $cart_cost,
						'currency' => $currency,
						'transaction_amount_currency' => $cart_cost,
						'tt_date' => time(),
						'commission_amount' => $totalAmount,
						'stat' => $states[$status_arr[$status_to]],
						'secret_key' => $secret_key,
					];
					
					$output = $switips_core->sendRequestPost($args);
				}
			}
		}
	}
	
	/**
	 * @param $product_id
	 * @return array
	 */
	function fn_switips_get_product_categories($product_id) {
		return db_get_array(
			'SELECT category_id FROM ?:products_categories'
			. ' WHERE product_id = ?i',
			$product_id
		);
	}
	
	/**
	 * @param $entity_id
	 * @param $entity_type
	 * @return array
	 */
	function fn_switips_get_entity_commission($entity_id, $entity_type) {
		// Define fields that should be retrieved
		$fields = [
			'?:switips_conditions.*',
			'?:switips_commissions.*'
		];
		
		$condition = $join = '';
		
		$join .= db_quote(
			' LEFT JOIN ?:switips_commissions ON ?:switips_commissions.commission_id = ?:switips_conditions.commission_id'
		);
		
		$condition .= db_quote(' ?:switips_conditions.entity_id IN (?n) AND ?:switips_conditions.type = ?s AND ?:switips_commissions.status = "A"', $entity_id, $entity_type);
		
		$fields = implode(', ', $fields);
		$commission = db_get_array('SELECT ?p FROM ?:switips_conditions ?p WHERE ?p',
			$fields,
			$join,
			$condition
		);
		
		/*fn_print_r($commission);
		
		die();*/
		
		return $commission;
	}
	
	/**
	 * @param $commissions
	 * @return array
	 */
	function fn_switips_get_entity_amount($commissions) {
		$entityAmount = array();
		
		foreach($commissions as $commission) {
			if($commission['amount']) {
				
				if (strstr($commission['amount'], '%')) {
					$amountValue = trim(str_replace('%', '', $commission['amount']));
					$entityAmount['percent'][] = $amountValue;
				}
				else {
					$entityAmount['absolute'][] = $commission['amount'];
				}
			}
		}
		
		return $entityAmount;
	}
	
	function fn_switips_get_item($order_id) {
		return db_get_row("SELECT * FROM ?:switips_items WHERE ?:switips_items.order = ?i", $order_id);
	}
	
	/**
	 * @param $item_data
	 * @return mixed
	 */
	function fn_switips_set_item($item_data) {
		return db_query('INSERT INTO ?:switips_items ?e', $item_data);
	}
	
	function fn_switips_set_status($status_data) {
		return db_query('INSERT INTO ?:switips_statuses ?e', $status_data);
	}
	
	function fn_switips_get_uid($item) {
		return $item['user_id'];
	}
	
	function fn_switips_order_status()
	{
		$order_statuses = fn_get_statuses(STATUSES_ORDER);
		$result = array();
		
		if (!empty($order_statuses)) {
			foreach ($order_statuses as $field) {
				if($field['status_id'] !== null) {
					$result[$field['status_id']] = $field['description'];
				}
			}
		}
		
		return $result;
	}
	
	function fn_settings_variants_addons_switips_order_status_new()
	{
		return fn_switips_order_status();
	}
	
	function fn_settings_variants_addons_switips_order_status_paid()
	{
		return fn_switips_order_status();
	}
	
	function fn_settings_variants_addons_switips_order_status_canceled()
	{
		return fn_switips_order_status();
	}
	
	function fn_settings_variants_addons_switips_order_status_confirmed()
	{
		return fn_switips_order_status();
	}
	
	/**
	 * Get switips orders
	 *
	 * @param array   $params          Search params
	 * @param int     $items_per_page  Items per page
	 * @param string  $lang_code       Language code
	 *
	 * @return array list of orders in first element, filtered parameters in second
	 */
	function fn_get_switips_orders($params, $items_per_page = 0, $lang_code = CART_LANGUAGE)
	{
		// Init filter
		$params = LastView::instance()->update('switips_orders', $params);
		
		// Set default values to input params
		$default_params = [
			'page'           => 1,
			'items_per_page' => $items_per_page,
			'get_hidden'     => true,
			'extend'         => []
		];
		
		$params = array_merge($default_params, $params);
		
		
		// Define fields that should be retrieved
		$fields = [
			'?:switips_items.*',
			'?:switips_statuses.*',
			'?:status_descriptions.description',
		];
		
		// Define sort fields
		$sortings = [
			'status' => '?:switips_statuses.status',
			'createdon'   => '?:switips_statuses.createdon',
			'amount'     => '?:switips_statuses.amount',
			'cost' => '?:switips_statuses.cost',
			'user_id'   => '?:switips_items.user_id',
			'order'     => '?:switips_items.order',
			'status_id'     => '?:switips_statuses.status_id',
		];
		
		$condition = $join = $group = '';
		
		$join .= db_quote(
			' LEFT JOIN ?:switips_items ON ?:switips_items.item_id = ?:switips_statuses.item'.
			' LEFT JOIN ?:status_descriptions ON ?:status_descriptions.status_id = ?:switips_statuses.status'.
			' AND ?:status_descriptions.lang_code = ?s',
			$lang_code
		);
		
		$sorting = db_sort($params, $sortings, 'status_id', 'desc');
		
		$limit = '';
		if (!empty($params['items_per_page'])) {
			$params['total_items'] = db_get_field(
				'SELECT COUNT(*) FROM ?:switips_statuses ?p WHERE 1 ?p ?p',
				$join,
				$condition,
				$group
			);
			$limit = db_paginate($params['page'], $params['items_per_page'], $params['total_items']);
		}
		
		
		$fields = implode(', ', $fields);
		$orders = db_get_hash_array('SELECT ?p FROM ?:switips_statuses ?p WHERE 1 ?p ?p ?p ?p',
			'status_id',
			$fields,
			$join,
			$condition,
			$group,
			$sorting,
			$limit
		);
		
		return array($orders, $params);
	}
	
	/**
	 * Get switips commissions
	 *
	 * @param array   $params          Search params
	 * @param int     $items_per_page  Items per page
	 * @param string  $lang_code       Language code
	 *
	 * @return array list of commissions in first element, filtered parameters in second
	 */
	function fn_get_switips_commissions($params, $items_per_page = 0, $lang_code = CART_LANGUAGE)
	{
		// Init filter
		$params = LastView::instance()->update('switips_commissions', $params);
		
		// Set default values to input params
		$default_params = [
			'page'           => 1,
			'items_per_page' => $items_per_page,
			'get_hidden'     => true,
			'extend'         => []
		];
		
		$params = array_merge($default_params, $params);
		
		
		// Define fields that should be retrieved
		$fields = [
			'?:switips_commissions.*',
			'?:switips_commission_descriptions.name',
		];
		
		// Define sort fields
		$sortings = [
			'amount' => '?:switips_commissions.amount',
			'status'   => '?:switips_commissions.status',
			'name'     => '?:switips_commission_descriptions.name',
		];
		
		$condition = $join = $group = '';
		
		$statuses = ['A'];
		if (!empty($params['get_hidden'])) {
			$statuses[] = 'H';
		}
		
		if (!empty($params['commission_id'])) {
			$condition .= db_quote(' AND ?:switips_commissions.commission_id IN (?n)', $params['commission_id']);
		}
		
		if (!empty($params['active'])) {
			$condition .= db_quote(
				' AND status IN (?a)',
				$statuses
			);
		}
		
		$join .= db_quote(
			' LEFT JOIN ?:switips_commission_descriptions ON ?:switips_commission_descriptions.commission_id = ?:switips_commissions.commission_id'
			. ' AND ?:switips_commission_descriptions.lang_code = ?s',
			$lang_code
		);
		
		$sorting = db_sort($params, $sortings, 'name', 'desc');
		
		$limit = '';
		if (!empty($params['items_per_page'])) {
			$params['total_items'] = db_get_field(
				'SELECT COUNT(*) FROM ?:switips_commissions ?p WHERE 1 ?p ?p',
				$join,
				$condition,
				$group
			);
			$limit = db_paginate($params['page'], $params['items_per_page'], $params['total_items']);
		}
		
		if (!empty($params['simple'])) {
			return db_get_hash_single_array(
				'SELECT ?:switips_commissions.commission_id, ?:switips_commission_descriptions.name FROM ?:switips_commissions ?p WHERE 1 ?p ?p ?p ?p',
				array('commission_id', 'name'),
				$join,
				$condition,
				$group,
				$sorting,
				$limit
			);
		} else {
			$fields = implode(', ', $fields);
			$commissions = db_get_hash_array('SELECT ?p FROM ?:switips_commissions ?p WHERE 1 ?p ?p ?p ?p',
				'commission_id',
				$fields,
				$join,
				$condition,
				$group,
				$sorting,
				$limit
			);
		}
		
		if (!empty($params['expand'])) {
			foreach ($commissions as $k => $v) {
				$commissions[$k]['conditions'] = !empty($v['conditions']) ? unserialize($v['conditions']) : array();
			}
		}
		
		return array($commissions, $params);
	}
	
	/**
	 * Get condition schema
	 *
	 * @return array schema of definite type
	 */
	function fn_switips_commission_get_schema()
	{
		static $schema = array();
		
		if (empty($schema)) {
			$schema = fn_get_schema('switips', 'conditions');
		}
		
		return $schema;
	}
	
	/**
	 * Validate product (convert to common format)
	 *
	 * @param array $product product data
	 * @return array converted product data
	 */
	function fn_switips_commission_validate_product($commission, $product)
	{
		if (!isset($product['product_id'])) {
			return array();
		}
		
		$options = array();
		
		if (!empty($commission['value']) && is_array($commission['value'])) {
			
			if (!empty($product['product_options'])) {
				
				foreach ($product['product_options'] as $key => $item) {
					if (isset($item['option_id']) && isset($item['value'])) {
						$options[$item['option_id']] = $item['value'];
						continue;
					}
					$options[$key] = $item;
				}
				
				$upd_product = array('product_options' => $options, 'product_id' => $product['product_id'], 'amount' => $product['amount']);
			} else {
				$upd_product = array('product_id' => $product['product_id'], 'amount' => $product['amount']);
			}
			foreach ($commission['value'] as $p_v) {
				if ($p_v['product_id'] == $upd_product['product_id']) {
					if (
						!isset($upd_product['product_options'], $p_v['product_options'])
						|| $upd_product['product_options'] == $p_v['product_options']
					) {
						if ($upd_product['amount'] >= $p_v['amount']) {
							$upd_product['amount'] = $p_v['amount'];
						}
					}
				}
			}
		} else {
			$upd_product = $product['product_id'];
		}
		
		return array($upd_product);
	}
	
	/**
	 * Get commission data
	 *
	 * @param int $commission_id commission ID
	 * @param string $lang_code code language
	 * @return array commission data
	 */
	function fn_get_switips_commission_data($commission_id, $lang_code = DESCR_SL)
	{
		$fields = [
			'p.*',
			'd.*'
		];
		
		$joins = [
			'switips_commission_descriptions' => db_quote(
				'LEFT JOIN ?:switips_commission_descriptions as d'
				. ' ON p.commission_id = d.commission_id AND d.lang_code = ?s',
				$lang_code
			)
		];
		
		$commission_data = db_get_row('SELECT ?p FROM ?:switips_commissions as p ?p WHERE p.commission_id = ?i',
			implode(', ', $fields),
			implode(' ', $joins),
			$commission_id
		);
		
		$condition_data = db_get_array('SELECT ?p FROM ?:switips_conditions as p WHERE p.commission_id = ?i',
			'p.*',
			$commission_id
		);
		
		if (!empty($condition_data)) {
			foreach ($condition_data as $condition) {
				$temp_conditions[$condition['type']][] = $condition['entity_id'];
			}
			unset($condition);
			
			$i = 0;
			foreach ($temp_conditions as $key => $condition) {
				$commission_data['conditions']['conditions'][$i]['condition'] = $key;
				$commission_data['conditions']['conditions'][$i]['value'] = implode(',', $condition);
				
				$i++;
			}
		}
		
		return $commission_data;
	}
	
	function fn_get_switips_commission_name($commission_id, $lang_code = DESCR_SL)
	{
		$commission_name = false;
		
		if (!empty($commission_id)) {
			$commission_name = db_get_field("SELECT name FROM ?:switips_commission_descriptions WHERE commission_id = ?i AND lang_code = ?s", $commission_id, $lang_code);
		}
		
		return $commission_name;
	}
	
	/**
	 * Update commission condition
	 *
	 * @param array $conditions conditions
	 * @param string $action update action
	 * @param string $field condition field to update
	 * @param string $value value to update field with
	 * @return boolean always true
	 */
	function fn_switips_commission_update_condition(&$conditions, $action, $field, $value)
	{
		foreach ($conditions as $k => $c) {
			if (!empty($c['conditions'])) {
				fn_switips_commission_update_condition($c['conditions'], $action, $field, $value);
			} elseif ($c['condition'] == $field) {
				if ($action == 'add') {
					$conditions[$k]['value'] .= (!empty($c['value']) ? ',' : '') . $value;
				} else {
					$conditions[$k]['value'] = preg_replace("/(\b{$value}\b[,]?[ ]?)/", '', $c['value']);
				}
			}
		}
		
		return true;
	}
	
	/**
	 * Delete commissions
	 *
	 * @param int|int[] $commission_ids - Сommission identifiers
	 */
	function fn_delete_switips_commissions($commission_ids)
	{
		if (!is_array($commission_ids)) {
			$commission_ids = array($commission_ids);
		}
		
		if (fn_allowed_for('ULTIMATE')) {
			foreach ($commission_ids as $commission_id => $commission) {
				if (!fn_check_company_id('commissions', 'commission_id', $commission)) {
					fn_set_notification('E', __('error'), __('access_denied'));
					unset($commission_ids[$commission_id]);
				}
			}
		}
		foreach ($commission_ids as $pr_id) {
			db_query('DELETE FROM ?:switips_commissions WHERE commission_id = ?i', $pr_id);
			db_query('DELETE FROM ?:switips_commission_descriptions WHERE commission_id = ?i', $pr_id);
		}
	}
	
	/**
	 * Updates commission
	 *
	 * @param array   $data          Сommission data
	 * @param int     $commission_id  Сommission identifier
	 * @param string  $lang_code     Language code (ru, en, etc)
	 *
	 * @return int Returns commission identifier
	 */
	function fn_update_switips_commission($data, $commission_id, $lang_code = DESCR_SL)
	{
		
		SecurityHelper::sanitizeObjectData('commission', $data);
		
		$data['conditions'] = empty($data['conditions']) ? array() : $data['conditions'];
		
		$data['amount'] = preg_replace(array('/[^0-9,\.%]/', '/,/'), array('', '.'), $data['amount']);
		if (strstr($data['amount'], '%')) {
			$data['amount'] = trim(str_replace(' ', '', $data['amount']));
		}
		if ((empty($data['amount']) && $data['amount'] != '')) {
			$data['amount'] = '0';
		}
		
		if (!empty($commission_id)) {
			db_query('UPDATE ?:switips_commissions SET ?u WHERE commission_id = ?i', $data, $commission_id);
			db_query(
				'UPDATE ?:switips_commission_descriptions SET ?u WHERE commission_id = ?i AND lang_code = ?s',
				$data,
				$commission_id,
				$lang_code
			);
		} else {
			$commission_id = $data['commission_id'] = db_query('REPLACE INTO ?:switips_commissions ?e', $data);
			
			foreach (Languages::getAll() as $data['lang_code'] => $_v) {
				db_query('REPLACE INTO ?:switips_commission_descriptions ?e', $data);
			}
		}
		
		db_query('DELETE FROM ?:switips_conditions WHERE commission_id = ?i', $commission_id);
		
		if(!empty($data['conditions']['conditions'])) {
			$conditions_array = array();
			
			$conditions = $data['conditions']['conditions'];
			
			foreach($conditions as $condition) {
				if(isset($condition['value'])) {
					$condition_value_array = explode(',', $condition['value']);
					
					foreach($condition_value_array as $condition_value) {
						
						$conditions_array['type'] = $condition['condition'];
						$conditions_array['entity_id'] = (int) $condition_value;
						$conditions_array['commission_id'] = $commission_id;
						
						db_query('REPLACE INTO ?:switips_conditions ?e', $conditions_array);
					}
					
					unset($conditions_array);
				}
			}
			
			unset($condition_value_array);
		}
		
		return $commission_id;
	}